package com.ozs.web.controller.accountmanagment;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.config.BaseConfig;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.config.ServerConfig;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseDeviceDynamicManagement;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.vo.BaseCameraManagementVo;
import com.ozs.service.entity.vo.BaseCameraResVo;
import com.ozs.service.entity.vo.MsgAlarmResp;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.BaseDeviceDynamicManagementService;
import com.ozs.service.service.BaseRailwayManagementService;
import com.ozs.service.service.MsgAlarmService;
import com.ozs.system.service.ISysDeptService;
import com.ozs.web.core.config.CaneraConfig;
import com.ozs.web.core.util.CameraUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.MediaType;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 相机台账管理表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-17
 */
@RestController
@RequestMapping("/service/baseCameraManagement")
@Slf4j
public class BaseCameraManagementController extends BaseController {
    @Resource
    BaseCameraManagementService baseCameraManagementService;
    @Resource
    BaseDeviceDynamicManagementService baseDeviceDynamicManagementService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private MsgAlarmService msgAlarmService;
    @Autowired
    private BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private ServerConfig serverConfig;
    @Autowired
    private CaneraConfig caneraConfig;

    /**
     * 相机台账管理分页
     *
     * @param baseCameraManagementVo
     * @return
     */
    @ApiOperation(value = "相机台账管理分页")
    @PostMapping("/list")
    public AjaxResult selectBaseAccountManagement(@RequestBody BaseCameraManagementVo baseCameraManagementVo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getLineDir()) && baseCameraManagementVo.getLineDir() != 0) {
            lw.eq(BaseCameraManagement::getLineDir, baseCameraManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseCameraManagementVo.getRailwayCode())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseCameraManagementVo.getRailwayCode());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseCameraManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseCameraManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseCameraManagementVo.getEndMile());
        }
        IPage<BaseCameraManagement> page = baseCameraManagementService.page(new Page<>(baseCameraManagementVo.getPageNum(), baseCameraManagementVo.getPageSize()), lw);
        if (!ObjectUtils.isEmpty(page) && !ObjectUtils.isEmpty(page.getRecords())) {
            List<BaseCameraManagement> dto1 = page.getRecords().stream().map(o -> {
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(o.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, o.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                SysDept sysDept = deptService.selectDeptById(o.getDeptId());
                o.setDeptName(sysDept.getDeptName());
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                String mils = AppendUtils.stringAppend(o.getInstallMile());
                o.setInstallMiles(mils);
                String end = AppendUtils.stringAppend(o.getEndMile());
                o.setEndMiles(end);
                String begin = AppendUtils.stringAppend(o.getBeginMile());
                o.setBeginMiles(begin);
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    /**
     * 删除相机台账管理信息
     *
     * @param accountManagementIds
     * @return
     */
    @DeleteMapping("/{accountManagementIds}")
    @ApiOperation("删除相机台账管理信息")
    @Transactional
    public AjaxResult removeAccountManagement(@PathVariable List<Long> accountManagementIds) {
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        for (Long accountManagementId : accountManagementIds) {
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(accountManagementId);
            baseCameraManagementService.removeById(accountManagementId);
            if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
                queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
            }
            baseDeviceDynamicManagementService.remove(queryWrapper);
        }
        return AjaxResult.success();
    }

    /**
     * 新增相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/saveAccountManagement")
    @ApiOperation("新增相机台账管理信息")
    @Transactional
    public AjaxResult saveDistributionModule(@RequestBody BaseCameraManagement baseCameraManagement) {
        int miles = AppendUtils.stringSplit(baseCameraManagement.getBeginMiles());
        baseCameraManagement.setBeginMile(miles);
        int install = AppendUtils.stringSplit(baseCameraManagement.getInstallMiles());
        baseCameraManagement.setInstallMile(install);
        int end = AppendUtils.stringSplit(baseCameraManagement.getEndMiles());
        baseCameraManagement.setEndMile(end);
        baseCameraManagement.setCreateBy(getUserId());
        baseCameraManagement.setUpdateBy(getUserId());
        BaseDeviceDynamicManagement baseDynamicManagement = new BaseDeviceDynamicManagement();
        baseDynamicManagement.setCameraCode(baseCameraManagement.getCameraCode());
        baseDynamicManagement.setUpdateBy(getUserId());
        baseDynamicManagement.setCreateBy(getUserId());
        if (baseCameraManagementService.save(baseCameraManagement) && baseDeviceDynamicManagementService.save(baseDynamicManagement)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    /**
     * 修改相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PutMapping("/editAccountManagement")
    @ApiOperation("修改相机台账管理信息")
    public AjaxResult editAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        int miles = AppendUtils.stringSplit(baseCameraManagement.getBeginMiles());
        baseCameraManagement.setBeginMile(miles);
        int install = AppendUtils.stringSplit(baseCameraManagement.getInstallMiles());
        baseCameraManagement.setInstallMile(install);
        int end = AppendUtils.stringSplit(baseCameraManagement.getEndMiles());
        baseCameraManagement.setEndMile(end);
        baseCameraManagement.setUpdateBy(getUserId());
        if (baseCameraManagementService.updateById(baseCameraManagement)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据相机ID获取相机台账管理信息")
    public AjaxResult getInfo(@PathVariable Long id) {
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getById(id);
        LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            queryWrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
        SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
        baseCameraManagement.setDeptName(sysDept.getDeptName());
        baseCameraManagement.setRailwayName(baseRailwayManagement.getRailwayName());
        String mils = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        baseCameraManagement.setInstallMiles(mils);
        String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
        baseCameraManagement.setEndMiles(end);
        String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
        baseCameraManagement.setBeginMiles(begin);
        return AjaxResult.success(baseCameraManagement);
    }

    @GetMapping("/getDate/{cameraCode}")
    @ApiOperation("视频上方显示的数据")
    public AjaxResult getDate(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
        LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
        LambdaQueryWrapper<BaseDeviceDynamicManagement> queryWrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getCameraCode())) {
            queryWrapper.eq(BaseDeviceDynamicManagement::getCameraCode, baseCameraManagement.getCameraCode());
        }
        BaseDeviceDynamicManagement baseDeviceDynamicManagement = baseDeviceDynamicManagementService.getOne(queryWrapper);
        List<Object> list = new ArrayList<>();
        list.add(baseRailwayManagement.getRailwayName());
        list.add(AppendUtils.stringAppend(baseCameraManagement.getInstallMile()));
        list.add(baseCameraManagement.getCameraCode());
        list.add(baseDeviceDynamicManagement.getElectricity());
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format1 = dateFormat1.format(new Date());
        list.add(format1);
        return AjaxResult.success(list);
    }

    /**
     * 台账管理导入
     *
     * @param file
     * @param updateSupport
     * @return
     * @throws Exception
     */
    @ApiOperation("相机台账管理信息导入")
    @PostMapping("/importBaseAccountManagement")
    public AjaxResult importBaseAccountManagement(MultipartFile file, boolean updateSupport) throws Exception {
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<BaseCameraManagement>(BaseCameraManagement.class);
        List<BaseCameraManagement> AccountManageList = util.importExcel(file.getInputStream());
        String message = baseCameraManagementService.importBaseAccountManagement(AccountManageList, updateSupport);
        return AjaxResult.success(message);
    }

    /**
     * 导出台账管理
     *
     * @param response
     */
    @ApiOperation("导出相机台账管理信息")
    @PostMapping("/exportBaseAccountManagement")
    public void exportBaseAccountManagement(HttpServletResponse response, @RequestBody BaseCameraManagementVo baseAccountManagementVo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getLineDir())) {
            lw.eq(BaseCameraManagement::getLineDir, baseAccountManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseAccountManagementVo.getRailwayCode())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseAccountManagementVo.getRailwayCode());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseAccountManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseAccountManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseAccountManagementVo.getEndMile());
        }
        List<BaseCameraManagement> list = baseCameraManagementService.list(lw);
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<>(BaseCameraManagement.class);
        util.exportExcel(response, list, "台账管理相机数据");
    }


    @ApiOperation("相机台账管理信息模板下载")
    @GetMapping("/downloaExcel")
    public void downloadZip(HttpServletResponse response) {
        try {
            InputStream resourceAsStream = this.getClass().getResourceAsStream("/template/provisionalPlan.xlsx");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, "相机台账管理信息模板.xlsx");
            FileUtils.writeBytesByInput(resourceAsStream, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    @ApiOperation("查询录像")
    @GetMapping("/api/records/{cameraCode}")
    public AjaxResult apiRecords(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel()));
    }

    @ApiOperation("播放回放")
    @PostMapping("/api/invite")
    public AjaxResult apiInvite(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        return success(CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime(), caneraConfig.getMappingUrl()));
    }

    @ApiOperation("播放回放")
    @PostMapping("/api/play")
    public AjaxResult play(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.historyPlayListStr(one.getChannel(), vo.getStartTime(), vo.getEntTime()));
//        return success(CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime()));
    }

    @ApiOperation("视频服务设备数量")
    @GetMapping("/totalNumberOfDevices")
    public AjaxResult totalNumberOfDevices() {
        long count = baseDeviceDynamicManagementService.count();
        return success(count);
    }

    @ApiOperation("视频服务树形结构")
    @GetMapping("/cameraTree")
    public AjaxResult cameraTree(@RequestParam(value = "name", required = false) String name) {
        return success(baseDeviceDynamicManagementService.cameraTree(name));
    }

    @GetMapping(value = "/getAlarm/{cameraCode}")
    @ApiOperation("根据视频ID获取详细信息")
    public AjaxResult getAlarm(@PathVariable String cameraCode) {
        LambdaQueryWrapper<MsgAlarm> queryWrapper = new LambdaQueryWrapper<MsgAlarm>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            queryWrapper.eq(MsgAlarm::getCameraCode, cameraCode);
            queryWrapper.orderByDesc(MsgAlarm::getCreateTime);
        }
        List<MsgAlarm> msgAlarmList = msgAlarmService.list(queryWrapper);
        MsgAlarm msgAlarm = msgAlarmList.get(0);
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(cameraCode)) {
            lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
        LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
        if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
            wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
        }
        BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
        MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
        BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
        msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
        msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
        String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
        String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
        String endMile = begin + end;
        msgAlarmResp.setBeginMile(endMile);
        String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        msgAlarmResp.setAlarmMile(mile);
        msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
        msgAlarmResp.setLineDir(baseCameraManagement.getLineDir());
        msgAlarmResp.setAlarmMile(baseCameraManagement.getInstallMile().toString());
        return AjaxResult.success(msgAlarmResp);
    }
}

